define("ember-cli-simple-store", ["ember-cli-simple-store/index","exports"], function(__index__, __exports__) {
  "use strict";
  Object.keys(__index__).forEach(function(key){
    __exports__[key] = __index__[key];
  });
});

define('ember-cli-simple-store/initializers/ember-cli-simple-store', ['exports', 'ember-cli-simple-store/store'], function (exports, Store) {

    'use strict';

    exports['default'] = function(container, application) {
        application.register('store:main', Store['default']);
        application.inject('controller', 'store', 'store:main');
        application.inject('route', 'store', 'store:main');
    }

});
define('ember-cli-simple-store/model', ['exports', 'ember'], function (exports, Ember) {

    'use strict';

    function factory(obj) {
        return obj.get("constructor.ClassMixin.ownerConstructor");
    }

    function attrs(obj) {
        var all = [];
        factory(obj).eachComputedProperty(function (key, meta) {
            if (meta.isAttribute) {
                all.push(key);
            }
        });
        return all;
    }

    function clone(obj) {
        var copy = {};
        factory(obj).eachComputedProperty(function (key, meta) {
            if (meta.isAttribute) {
                copy[key] = obj.get(key);
            }
        });
        return copy;
    }

    var attr = function() {
        var meta = {isAttribute: true};
        return function(key, value) {
            var data = this.get("_data") || {};
            var dirty = this.get("_dirty") || {};
            if (arguments.length === 2) {
                if (!this.get("isDirty") && !this.get("isPrimed")) {
                    var oldState = clone(this);
                    this.set("_oldState", oldState);
                }
                var primed = value === "" && data[key] === undefined;
                if(!primed) {
                    this.set("isPrimed", true);
                    dirty["%@:isDirty".fmt(key)] = true;
                    data[key] = value;
                }
            }
            return data[key];
        }.property("_data").meta(meta);
    };

    var Model = Ember['default'].Object.extend({
        init: function() {
            this._super();
            this._reset();
            this._setup();
            this.set("_data", {});
            this.set("_oldState", clone(this));
        },
        rollback: function() {
            var oldState = this.get("_oldState");
            for(var key in oldState){
                this.set(key, oldState[key]);
            }
            this._reset();
        },
        save: function() {
            var oldState = clone(this);
            this.set("_oldState", oldState);
            this._reset();
        },
        _reset: function() {
            this.set("isPrimed", false);
            this.set("_dirty", {});
        },
        _setup: function() {
            var self = this;
            var attributes = attrs(this);
            attributes.forEach(function(attrName) {
                var dynamicKey = "%@IsDirty".fmt(attrName);
                Ember['default'].defineProperty(self, dynamicKey, Ember['default'].computed(function() {
                    var current = this.get(attrName);
                    var original = this.get("_oldState." + attrName);
                    var dirty = this.get("_dirty");
                    var dirtyKey = "%@:isDirty".fmt(attrName);
                    return original === current ? undefined : dirty[dirtyKey];
                }).property("_dirty", "" + attrName));
            });
            var modelIsDirtyAttrs = [];
            attributes.forEach(function(attr) {
                modelIsDirtyAttrs.push(attr + "IsDirty");
            });
            Ember['default'].defineProperty(this, "isDirty", Ember['default'].computed(function() {
                var modelAttrs = modelIsDirtyAttrs.filter(function(attr){
                    return self.get(attr) === true;
                });
                return modelAttrs.length > 0;
            }).property("" + modelIsDirtyAttrs));
        }
    });

    exports.attr = attr;
    exports.Model = Model;

});
define('ember-cli-simple-store/store', ['exports', 'ember'], function (exports, Ember) {

    'use strict';

    function buildRecord(type, data, store) {
        var containerKey = "model:" + type;
        var factory = store.container.lookupFactory(containerKey);

        Ember['default'].assert("No model was found for type: " + type, factory);

        var record = factory.create(data);
        var id = data.id;
        identityMapForType(type, store)[id] = record;
        arrayForType(type, store).pushObject(record);
        return record;
    }

    function arrayForType(type, store) {
        var all = store.get("array");
        var models = all[type] || [];
        all[type] = models;
        return models;
    }

    function identityMapForType(type, store) {
        var typeIdentityMap = store.get("identityMap");
        var idIdentityMap = typeIdentityMap[type] || {};
        typeIdentityMap[type] = idIdentityMap;
        return idIdentityMap;
    }

    var Store = Ember['default'].Object.extend({
        init: function() {
            this.set("identityMap", {});
            this.set("array", {});
        },
        clear: function(type) {
            if(type === undefined) {
                this.init();
            }
            delete this.get("identityMap")[type];
            arrayForType(type, this).clear();
        },
        push: function(type, data) {
            var record = this._findById(type, data.id);
            if (record) {
                record.setProperties(data);
            } else {
                record = buildRecord(type, data, this);
            }
            return record;
        },
        remove: function(type, id) {
            var record = this._findById(type, id);
            if (record) {
                delete this.get("identityMap")[type][record.id];
                arrayForType(type, this).removeObject(record);
            }
        },
        find: function(type, options) {
            if (typeof options === "undefined") {
                return this._findAll(type);
            }
            if (options instanceof Function) {
                var computed_keys = arguments[2];
                Ember['default'].assert("No computed keys found for the filter by function", computed_keys);
                return this._findWithFilterFunc(type, options, computed_keys);
            }
            if (typeof options === "object") {
                var params = Object.keys(options);

                Ember['default'].assert("No key was found in the filter options", params.length);

                var attr = params[0];
                var value = options[attr];
                return this._findWithFilter(type, attr, value);
            }
            return this._findByIdComputed(type, options);
        },
        _findById: function(type, id) {
            var identityMap = identityMapForType(type, this);
            return identityMap[id] || null;
        },
        _findAll: function(type) {
            return arrayForType(type, this);
        },
        _findWithFilter: function(type, filter_attr, filter_value) {
            var computed_string = "source.@each." + filter_attr;
            return Ember['default'].ArrayProxy.extend({
              content: function () {
                var filter_value = this.get("filter_value");
                return this.get("source").filterBy(filter_attr, filter_value);
              }.property(computed_string)
            }).create({
              filter_value: filter_value,
              source: this._findAll(type)
            });
        },
        _findWithFilterFunc: function(type, filter_func, computed_keys) {
            var attributes = [];
            computed_keys.forEach(function(computed_key) {
                attributes.push("source.@each." + computed_key);
            });
            return Ember['default'].ArrayProxy.extend({
              content: function () {
                var filter_func = this.get("filter_func");
                return this.get("source").filter(filter_func);
              }.property("" + attributes)
            }).create({
              filter_func: filter_func,
              source: this._findAll(type)
            });
        },
        _coerceId: function(id) {
            var numberId = parseInt(id, 10);
            if (numberId && numberId.toString().length === id.toString().length) {
                return numberId;
            }
            return id;
        },
        _findByIdComputed: function(type, id) {
            var store = this;
            var actualId = this._coerceId(id);
            return Ember['default'].ObjectProxy.extend({
                content: function() {
                    var filter_value = this.get("filter_value");
                    return this.get("source").filterBy("id", filter_value).objectAt(0);
                }.property("source.[]")
            }).create({
                filter_value: actualId,
                source: this._findAll(type),
                init: function () {
                    var model = store.container.lookup("model:" + type);
                    for(var method in model) {
                        if(typeof model[method] === "function") {
                            if(!this[method]) {
                                this.proxyMethod(method);
                            }
                        }
                    }
                },
                proxyMethod: function(method) {
                    this[method] = function() {
                        var content = this.get("content");
                        return content[method].apply(content, arguments);
                    };
                }
            });
        },
        findOne: function(type) {
            var store = this;
            return Ember['default'].ObjectProxy.extend({
                content: function() {
                    return this.get("source").objectAt(0);
                }.property("source.[]")
            }).create({
                source: this._findAll(type),
                init: function () {
                    var model = store.container.lookup("model:" + type);
                    for(var method in model) {
                        if(typeof model[method] === "function") {
                            if(!this[method]) {
                                this.proxyMethod(method);
                            }
                        }
                    }
                },
                proxyMethod: function(method) {
                    this[method] = function() {
                        var content = this.get("content");
                        return content[method].apply(content, arguments);
                    };
                }
            });
        }
    });

    exports['default'] = Store;

});
define("ember-cli-simple-validation", ["ember-cli-simple-validation/index","exports"], function(__index__, __exports__) {
  "use strict";
  Object.keys(__index__).forEach(function(key){
    __exports__[key] = __index__[key];
  });
});

define('ember-cli-simple-validation/components/validation-error-field', ['exports', 'ember'], function (exports, Ember) {

    'use strict';

    var ValidationErrorField = Ember['default'].Component.extend({
        tagName: "span",
        classNameBindings: ["visible"],
        attributeBindings: ["className", "model", "field", "validation", "submitted", "delayed", "index"],
        visible: function() {
            var index = this.get("index");
            var isDirty = this.get("isDirty");
            var delayed = this.get("delayed");
            var submitted = this.get("submitted");
            var className = this.get("className");
            var validator = this.get("validation");
            var validation = index >= 0 ? this.get("targetObject.%@1%@2Validation".fmt(validator, index)) : validator;
            var condition = delayed === true ? !validation && submitted : !validation && (isDirty || submitted);
            if(condition) {
                return className;
            }
            return className ? "hidden %@".fmt(className) : "hidden";
        }.property("validation", "isDirty", "submitted", "fieldName"),
        initialize: function() {
            var field = this.get("field");
            Ember['default'].Binding.from("model.%@".fmt(field)).to("fieldName").connect(this);
            Ember['default'].Binding.from("model.%@IsDirty".fmt(field)).to("isDirty").connect(this);
        }.on("init")
    });

    exports['default'] = ValidationErrorField;

});
define('ember-cli-simple-validation/mixins/validate', ['exports', 'ember'], function (exports, Ember) {

    'use strict';

    var dynamicEachAttrs = [];

    function factory(mixin) {
        return mixin.get("constructor.ClassMixin.ownerConstructor");
    }

    function eachAttrs(mixin) {
        var attributes = [];
        dynamicEachAttrs = [];
        factory(mixin).eachComputedProperty(function(field, meta) {
            if (meta.validateEach) {
                dynamicEachAttrs.push(field);
                attributes.push({field: field, options: meta.options, fieldName: meta.fieldName});
            }
        });
        return attributes;
    }

    function attrs(mixin) {
        var attributes = [];
        factory(mixin).eachComputedProperty(function(field) {
            if (field.indexOf("Validation") > 0) {
                attributes.push(field);
            }
        });
        dynamicEachAttrs.forEach(function(field) {
            mixin.get("model").forEach(function(model, index) {
                attributes.push("%@1%@2Validation".fmt(field, index));
            });
        });
        return attributes;
    }

    function run(value, options, context) {
        context.notifyPropertyChange("valid");
        if (options instanceof RegExp) {
            return options.test(value);
        }
        if (options instanceof Function) {
            return options.apply(context, arguments);
        }
        if (typeof value === "boolean") {
            return value;
        }
        if (typeof value === "string") {
            return value.trim().length > 0;
        }
        return value;
    }

    var ValidationMixin = Ember['default'].Mixin.create({
        valid: function() {
            var self = this;
            var result = true;
            attrs(this).forEach(function(attr) {
                result = self.get(attr) && result;
            });
            return result;
        }.property(),
        each: function() {
            var self = this;
            this.notifyPropertyChange("valid");
            eachAttrs(this).forEach(function(attr) {
                self.get("model").forEach(function(obj, index) {
                    Ember['default'].defineProperty(self, "%@1%@2Validation".fmt(attr.field, index), Ember['default'].computed(function() {
                        if(self.get("model").objectAt(index)) {
                            var value = self.get("model").objectAt(index).getWithDefault(attr.fieldName, "");
                            return run(value, attr.options, self, index);
                        }
                    }).property("model.@each." + attr.fieldName));
                });
            });
        }.observes("model.@each.isDone")
    });

    var validate = function(field, options) {
        return function() {
            var value = this.getWithDefault(field, "");
            return run(value, options, this);
        }.property(field);
    };

    var validateEach = function(fieldName, options) {
        return function() {
            return true;
        }.property().meta({validateEach: true, options: options, fieldName: fieldName});
    };

    exports.ValidationMixin = ValidationMixin;
    exports.validate = validate;
    exports.validateEach = validateEach;

});//# sourceMappingURL=addons.map