define('dummy/tests/acceptance/form-validation-test', ['ember', 'dummy/tests/helpers/start-app', 'qunit'], function (Ember, startApp, qunit) {

    'use strict';

    var application;

    var SAVE_BUTTON = "button.save";
    var NAME_ERROR_FIELD = ".name-parent-div span";
    var ZIPCODE_ERROR_FIELD = ".zipcode-parent-div span";
    var EMAIL_ERROR_FIELD = ".email-parent-div span";
    var RANDO_ERROR_FIELD = ".rando-parent-div span";
    var RADEO_ERROR_FIELD = ".radeo-parent-div span";
    var THING_ERROR_FIELD = ".thing-parent-div span";
    var TOS_ERROR_FIELD = ".tos-parent-div span";

    var NAME_INPUT = ".name-parent-div input";
    var ZIPCODE_INPUT = ".zipcode-parent-div input";
    var EMAIL_INPUT = ".email-parent-div input";
    var RANDO_INPUT = ".rando-parent-div input";
    var RADEO_BUTTON_ONE = ".radio-button-one input";
    var RADEO_BUTTON_TWO = ".radio-button-two input";
    var THING_SELECTBOX = ".thing-parent-div select";
    var TOS_CHECKBOX = ".tos-parent-div input";

    var VALID_NAME = "x";
    var VALID_ZIPCODE = "52601";
    var VALID_EMAIL = "hi@hi.com";
    var VALID_RANDO = "1";
    var VALID_THING = 1;

    qunit.module("Acceptance: Form Validation", {
        setup: function setup() {
            application = startApp['default']();
        },
        teardown: function teardown() {
            Ember['default'].run(application, "destroy");
        }
    });

    qunit.test("clicking save will transition to success when each field is valid", function (assert) {
        visit("/");
        fillIn(NAME_INPUT, VALID_NAME);
        fillIn(ZIPCODE_INPUT, VALID_ZIPCODE);
        fillIn(EMAIL_INPUT, VALID_EMAIL);
        fillIn(RANDO_INPUT, VALID_RANDO);
        fillIn(THING_SELECTBOX, VALID_THING);
        click(TOS_CHECKBOX);
        click(RADEO_BUTTON_ONE);
        click(SAVE_BUTTON);
        andThen(function () {
            assert.equal(currentURL(), "/success");
        });
    });

    qunit.test("clicking save will not transition to success when only the last field is valid", function (assert) {
        visit("/");
        click(TOS_CHECKBOX);
        click(SAVE_BUTTON);
        andThen(function () {
            assert.equal(currentURL(), "/");
        });
        fillIn(NAME_INPUT, VALID_NAME);
        fillIn(ZIPCODE_INPUT, VALID_ZIPCODE);
        fillIn(EMAIL_INPUT, VALID_EMAIL);
        fillIn(RANDO_INPUT, VALID_RANDO);
        fillIn(THING_SELECTBOX, VALID_THING);
        click(RADEO_BUTTON_ONE);
        click(SAVE_BUTTON);
        andThen(function () {
            assert.equal(currentURL(), "/success");
        });
    });

    qunit.test("string based inputs with default validation will not accept empty string as valid", function (assert) {
        visit("/");
        andThen(function () {
            assert.equal(find(NAME_ERROR_FIELD).hasClass("hidden"), true);
        });
        fillIn(NAME_INPUT, " ");
        andThen(function () {
            assert.equal(find(NAME_ERROR_FIELD).hasClass("hidden"), false);
        });
        fillIn(NAME_INPUT, VALID_NAME);
        andThen(function () {
            assert.equal(find(NAME_ERROR_FIELD).hasClass("hidden"), true);
        });
    });

    qunit.test("clicking save will fire each validation", function (assert) {
        visit("/");
        andThen(function () {
            assert.equal(currentURL(), "/");
            assert.equal(find(NAME_ERROR_FIELD).hasClass("hidden"), true);
            assert.equal(find(ZIPCODE_ERROR_FIELD).hasClass("hidden"), true);
            assert.equal(find(EMAIL_ERROR_FIELD).hasClass("hidden"), true);
            assert.equal(find(RANDO_ERROR_FIELD).hasClass("hidden"), true);
            assert.equal(find(RADEO_ERROR_FIELD).hasClass("hidden"), true);
            assert.equal(find(THING_ERROR_FIELD).hasClass("hidden"), true);
            assert.equal(find(TOS_ERROR_FIELD).hasClass("hidden"), true);
        });
        click(SAVE_BUTTON);
        andThen(function () {
            assert.equal(currentURL(), "/");
            assert.equal(find(NAME_ERROR_FIELD).hasClass("hidden"), false);
            assert.equal(find(ZIPCODE_ERROR_FIELD).hasClass("hidden"), false);
            assert.equal(find(EMAIL_ERROR_FIELD).hasClass("hidden"), false);
            assert.equal(find(RANDO_ERROR_FIELD).hasClass("hidden"), false);
            assert.equal(find(RADEO_ERROR_FIELD).hasClass("hidden"), false);
            assert.equal(find(THING_ERROR_FIELD).hasClass("hidden"), false);
            assert.equal(find(TOS_ERROR_FIELD).hasClass("hidden"), false);
        });
    });

    qunit.test("component will render with the correct text and css class", function (assert) {
        visit("/");
        click(SAVE_BUTTON);
        andThen(function () {
            assert.equal(currentURL(), "/");
            assert.equal(find(NAME_ERROR_FIELD).hasClass("red"), true);
            assert.equal(find(EMAIL_ERROR_FIELD).hasClass("blue"), true);
            assert.equal(find(NAME_ERROR_FIELD).text(), "invalid name");
            assert.equal(find(ZIPCODE_ERROR_FIELD).text(), "invalid zipcode");
            assert.equal(find(EMAIL_ERROR_FIELD).text(), "invalid email");
            assert.equal(find(RANDO_ERROR_FIELD).text(), "invalid rando");
            assert.equal(find(RADEO_ERROR_FIELD).text(), "invalid radeo");
            assert.equal(find(THING_ERROR_FIELD).text(), "invalid thing");
            assert.equal(find(TOS_ERROR_FIELD).text(), "invalid tos");
        });
    });

    qunit.test("when inputs are showing errors they turn hidden when field is correctly entered", function (assert) {
        visit("/");
        andThen(function () {
            assert.equal(find(NAME_ERROR_FIELD).hasClass("hidden"), true);
            assert.equal(find(ZIPCODE_ERROR_FIELD).hasClass("hidden"), true);
            assert.equal(find(EMAIL_ERROR_FIELD).hasClass("hidden"), true);
            assert.equal(find(RANDO_ERROR_FIELD).hasClass("hidden"), true);
            assert.equal(find(RADEO_ERROR_FIELD).hasClass("hidden"), true);
            assert.equal(find(THING_ERROR_FIELD).hasClass("hidden"), true);
            assert.equal(find(TOS_ERROR_FIELD).hasClass("hidden"), true);
        });
        click(SAVE_BUTTON);
        fillIn(NAME_INPUT, VALID_NAME);
        andThen(function () {
            assert.equal(find(NAME_ERROR_FIELD).hasClass("hidden"), true);
            assert.equal(find(ZIPCODE_ERROR_FIELD).hasClass("hidden"), false);
            assert.equal(find(EMAIL_ERROR_FIELD).hasClass("hidden"), false);
            assert.equal(find(RANDO_ERROR_FIELD).hasClass("hidden"), false);
            assert.equal(find(RADEO_ERROR_FIELD).hasClass("hidden"), false);
            assert.equal(find(THING_ERROR_FIELD).hasClass("hidden"), false);
            assert.equal(find(TOS_ERROR_FIELD).hasClass("hidden"), false);
        });
        fillIn(ZIPCODE_INPUT, VALID_ZIPCODE);
        andThen(function () {
            assert.equal(find(NAME_ERROR_FIELD).hasClass("hidden"), true);
            assert.equal(find(ZIPCODE_ERROR_FIELD).hasClass("hidden"), true);
            assert.equal(find(EMAIL_ERROR_FIELD).hasClass("hidden"), false);
            assert.equal(find(RANDO_ERROR_FIELD).hasClass("hidden"), false);
            assert.equal(find(RADEO_ERROR_FIELD).hasClass("hidden"), false);
            assert.equal(find(THING_ERROR_FIELD).hasClass("hidden"), false);
            assert.equal(find(TOS_ERROR_FIELD).hasClass("hidden"), false);
        });
        fillIn(EMAIL_INPUT, VALID_EMAIL);
        andThen(function () {
            assert.equal(find(NAME_ERROR_FIELD).hasClass("hidden"), true);
            assert.equal(find(ZIPCODE_ERROR_FIELD).hasClass("hidden"), true);
            assert.equal(find(EMAIL_ERROR_FIELD).hasClass("hidden"), true);
            assert.equal(find(RANDO_ERROR_FIELD).hasClass("hidden"), false);
            assert.equal(find(RADEO_ERROR_FIELD).hasClass("hidden"), false);
            assert.equal(find(THING_ERROR_FIELD).hasClass("hidden"), false);
            assert.equal(find(TOS_ERROR_FIELD).hasClass("hidden"), false);
        });
        fillIn(RANDO_INPUT, VALID_RANDO);
        andThen(function () {
            assert.equal(find(NAME_ERROR_FIELD).hasClass("hidden"), true);
            assert.equal(find(ZIPCODE_ERROR_FIELD).hasClass("hidden"), true);
            assert.equal(find(EMAIL_ERROR_FIELD).hasClass("hidden"), true);
            assert.equal(find(RANDO_ERROR_FIELD).hasClass("hidden"), true);
            assert.equal(find(RADEO_ERROR_FIELD).hasClass("hidden"), false);
            assert.equal(find(THING_ERROR_FIELD).hasClass("hidden"), false);
            assert.equal(find(TOS_ERROR_FIELD).hasClass("hidden"), false);
        });
        click(RADEO_BUTTON_ONE);
        andThen(function () {
            assert.equal(find(NAME_ERROR_FIELD).hasClass("hidden"), true);
            assert.equal(find(ZIPCODE_ERROR_FIELD).hasClass("hidden"), true);
            assert.equal(find(EMAIL_ERROR_FIELD).hasClass("hidden"), true);
            assert.equal(find(RANDO_ERROR_FIELD).hasClass("hidden"), true);
            assert.equal(find(RADEO_ERROR_FIELD).hasClass("hidden"), true);
            assert.equal(find(THING_ERROR_FIELD).hasClass("hidden"), false);
            assert.equal(find(TOS_ERROR_FIELD).hasClass("hidden"), false);
        });
        fillIn(THING_SELECTBOX, VALID_THING);
        andThen(function () {
            assert.equal(find(NAME_ERROR_FIELD).hasClass("hidden"), true);
            assert.equal(find(ZIPCODE_ERROR_FIELD).hasClass("hidden"), true);
            assert.equal(find(EMAIL_ERROR_FIELD).hasClass("hidden"), true);
            assert.equal(find(RANDO_ERROR_FIELD).hasClass("hidden"), true);
            assert.equal(find(RADEO_ERROR_FIELD).hasClass("hidden"), true);
            assert.equal(find(THING_ERROR_FIELD).hasClass("hidden"), true);
            assert.equal(find(TOS_ERROR_FIELD).hasClass("hidden"), false);
        });
        click(TOS_CHECKBOX);
        andThen(function () {
            assert.equal(find(NAME_ERROR_FIELD).hasClass("hidden"), true);
            assert.equal(find(ZIPCODE_ERROR_FIELD).hasClass("hidden"), true);
            assert.equal(find(EMAIL_ERROR_FIELD).hasClass("hidden"), true);
            assert.equal(find(RANDO_ERROR_FIELD).hasClass("hidden"), true);
            assert.equal(find(RADEO_ERROR_FIELD).hasClass("hidden"), true);
            assert.equal(find(THING_ERROR_FIELD).hasClass("hidden"), true);
            assert.equal(find(TOS_ERROR_FIELD).hasClass("hidden"), true);
        });
    });

    qunit.test("inputs with default validation start working without submit firing", function (assert) {
        visit("/");
        andThen(function () {
            assert.equal(find(NAME_ERROR_FIELD).hasClass("hidden"), true);
        });
        fillIn(NAME_INPUT, VALID_NAME);
        andThen(function () {
            assert.equal(find(NAME_ERROR_FIELD).hasClass("hidden"), true);
        });
        fillIn(NAME_INPUT, "");
        andThen(function () {
            assert.equal(find(NAME_ERROR_FIELD).hasClass("hidden"), false);
        });
        fillIn(NAME_INPUT, VALID_NAME);
        andThen(function () {
            assert.equal(find(NAME_ERROR_FIELD).hasClass("hidden"), true);
        });
    });

    qunit.test("inputs with regex validation start working without submit firing", function (assert) {
        visit("/");
        andThen(function () {
            assert.equal(find(ZIPCODE_ERROR_FIELD).hasClass("hidden"), true);
        });
        fillIn(ZIPCODE_INPUT, VALID_ZIPCODE);
        andThen(function () {
            assert.equal(find(ZIPCODE_ERROR_FIELD).hasClass("hidden"), true);
        });
        fillIn(ZIPCODE_INPUT, "");
        andThen(function () {
            assert.equal(find(ZIPCODE_ERROR_FIELD).hasClass("hidden"), false);
        });
        fillIn(ZIPCODE_INPUT, "3001");
        andThen(function () {
            assert.equal(find(ZIPCODE_ERROR_FIELD).hasClass("hidden"), false);
        });
        fillIn(ZIPCODE_INPUT, VALID_ZIPCODE);
        andThen(function () {
            assert.equal(find(ZIPCODE_ERROR_FIELD).hasClass("hidden"), true);
        });
    });

    qunit.test("inputs with regex validation will not work without submit when delay option set", function (assert) {
        visit("/");
        andThen(function () {
            assert.equal(find(EMAIL_ERROR_FIELD).hasClass("hidden"), true);
        });
        fillIn(EMAIL_INPUT, VALID_EMAIL);
        andThen(function () {
            assert.equal(find(EMAIL_ERROR_FIELD).hasClass("hidden"), true);
        });
        fillIn(EMAIL_INPUT, "");
        andThen(function () {
            assert.equal(find(EMAIL_ERROR_FIELD).hasClass("hidden"), true);
        });
        click(SAVE_BUTTON);
        andThen(function () {
            assert.equal(find(EMAIL_ERROR_FIELD).hasClass("hidden"), false);
        });
        fillIn(EMAIL_INPUT, VALID_EMAIL);
        andThen(function () {
            assert.equal(find(EMAIL_ERROR_FIELD).hasClass("hidden"), true);
        });
        fillIn(EMAIL_INPUT, "hi");
        andThen(function () {
            assert.equal(find(EMAIL_ERROR_FIELD).hasClass("hidden"), false);
        });
        fillIn(EMAIL_INPUT, VALID_EMAIL);
        andThen(function () {
            assert.equal(find(EMAIL_ERROR_FIELD).hasClass("hidden"), true);
        });
    });

    qunit.test("inputs with default validation will not trip validation when they are tabbed into", function (assert) {
        visit("/");
        andThen(function () {
            assert.equal(find(ZIPCODE_ERROR_FIELD).hasClass("hidden"), true);
        });
        triggerEvent(ZIPCODE_INPUT, "dblclick");
        triggerEvent(ZIPCODE_INPUT, "blur");
        andThen(function () {
            assert.equal(find(ZIPCODE_ERROR_FIELD).hasClass("hidden"), true);
        });
        fillIn(ZIPCODE_INPUT, "5003");
        triggerEvent(ZIPCODE_INPUT, "blur");
        andThen(function () {
            assert.equal(find(ZIPCODE_ERROR_FIELD).hasClass("hidden"), false);
        });
        fillIn(ZIPCODE_INPUT, "12345");
        triggerEvent(ZIPCODE_INPUT, "blur");
        andThen(function () {
            assert.equal(find(ZIPCODE_ERROR_FIELD).hasClass("hidden"), true);
        });
    });

    qunit.test("inputs with function validation start working without submit firing", function (assert) {
        visit("/");
        andThen(function () {
            assert.equal(find(RANDO_ERROR_FIELD).hasClass("hidden"), true);
        });
        fillIn(RANDO_INPUT, VALID_RANDO);
        andThen(function () {
            assert.equal(find(RANDO_ERROR_FIELD).hasClass("hidden"), true);
        });
        fillIn(RANDO_INPUT, "7");
        andThen(function () {
            assert.equal(find(RANDO_ERROR_FIELD).hasClass("hidden"), false);
        });
        fillIn(RANDO_INPUT, VALID_RANDO);
        andThen(function () {
            assert.equal(find(RANDO_ERROR_FIELD).hasClass("hidden"), true);
        });
    });

    qunit.test("number based inputs with default validation will validate correctly", function (assert) {
        visit("/");
        andThen(function () {
            assert.equal(find(RADEO_ERROR_FIELD).hasClass("hidden"), true);
        });
        click(SAVE_BUTTON);
        andThen(function () {
            assert.equal(find(RADEO_ERROR_FIELD).hasClass("hidden"), false);
        });
        click(RADEO_BUTTON_ONE);
        andThen(function () {
            assert.equal(find(RADEO_ERROR_FIELD).hasClass("hidden"), true);
        });
        click(RADEO_BUTTON_TWO);
        andThen(function () {
            assert.equal(find(RADEO_ERROR_FIELD).hasClass("hidden"), true);
        });
    });

    qunit.test("model bound selectbox with default validation will validate correctly", function (assert) {
        visit("/");
        andThen(function () {
            assert.equal(find(THING_ERROR_FIELD).hasClass("hidden"), true);
        });
        click(SAVE_BUTTON);
        andThen(function () {
            assert.equal(find(THING_ERROR_FIELD).hasClass("hidden"), false);
        });
        fillIn(THING_SELECTBOX, VALID_THING);
        andThen(function () {
            assert.equal(find(THING_ERROR_FIELD).hasClass("hidden"), true);
        });
        fillIn(THING_SELECTBOX, "");
        andThen(function () {
            assert.equal(find(THING_ERROR_FIELD).hasClass("hidden"), false);
        });
    });

});