define('dummy/tests/unit/validation-mixin-test', ['ember', 'qunit', 'ember-cli-simple-store/model', 'ember-cli-simple-validation/mixins/validate'], function (Ember, qunit, ember_cli_simple_store__model, validate) {

    'use strict';

    var Person = ember_cli_simple_store__model.Model.extend({
        name: ember_cli_simple_store__model.attr(),
        email: ember_cli_simple_store__model.attr()
    });

    var controller,
        model = Person.create({ name: "x", email: "hi@hi.com" });

    var FakeController = Ember['default'].Controller.extend(validate.ValidationMixin, {
        nameValidation: validate.validate("model.name"),
        emailValidation: validate.validate("model.email", /\S+@\S+\.\S+/),
        model: model
    });

    qunit.module("Unit: Validation Mixin", {
        setup: function setup() {
            controller = new FakeController();
        },
        teardown: function teardown() {
            controller = null;
        }
    });

    qunit.test("validation will fire when computed validation value is update", function (assert) {
        assert.equal(controller.get("nameValidation"), true);
        model.set("name", "");
        assert.equal(controller.get("nameValidation"), false);
        model.set("name", "x");
        assert.equal(controller.get("nameValidation"), true);
    });

    qunit.test("validate attr without regex option will trim the value by default", function (assert) {
        assert.equal(controller.get("nameValidation"), true);
        model.set("name", "");
        assert.equal(controller.get("nameValidation"), false);
        model.set("name", " ");
        assert.equal(controller.get("nameValidation"), false);
        model.set("name", "x");
        assert.equal(controller.get("nameValidation"), true);
    });

    qunit.test("validate attr with regex will validate using the given pattern", function (assert) {
        assert.equal(controller.get("emailValidation"), true);
        model.set("email", "");
        assert.equal(controller.get("emailValidation"), false);
        model.set("email", "hi@h");
        assert.equal(controller.get("emailValidation"), false);
        model.set("email", "hi@hi.com");
        assert.equal(controller.get("emailValidation"), true);
    });

});